package main

import (
	"log"

	"git.paulbsd.com/paulbsd/ipbl/src/config"
	"git.paulbsd.com/paulbsd/ipbl/src/database"
	"git.paulbsd.com/paulbsd/ipbl/src/ipbl"
	"git.paulbsd.com/paulbsd/ipbl/src/ipblws"
	_ "github.com/lib/pq"
	"github.com/robfig/cron"
)

//var version string

func main() {
	var cfg config.Config
	cfg.GetConfig()
	//cfg.Options.Version = version

	// Initialize database app context
	err := database.Init(&cfg)
	if err != nil {
		log.Fatalln(err)
	}
	defer cfg.Db.Close()

	// Handles IP with no reverse DNS
	go ipbl.ScanIP(&cfg)

	// Add cron task to handle them
	cr := cron.New()
	cr.AddFunc("0 * * * * *", func() {
		ipbl.ScanIP(&cfg)
	})
	cr.Start()

	// Run the ipbl web service
	err = ipblws.RunServer(&cfg)
	if err != nil {
		log.Fatalln(err)
	}
}
