package ipbl

import (
	"database/sql"
	"fmt"
	"log"
	"net"
	"time"

	"git.paulbsd.com/paulbsd/ipbl/src/config"
)

func (i *IP) UpdateRDNS() (result string) {
	res, err := net.LookupAddr(i.IP)
	if err != nil {
		return ""
	}
	return res[0]
}

func (i *IP) InsertIP(cfg *config.Config) (num int64, err error) {
	num, err = cfg.Db.Insert(i)
	return
}

func InsertIPBulk(cfg *config.Config, ips *[]IP) (numinserts int64, numfail int64, err error) {
	for _, ip := range *ips {
		num, err := cfg.Db.Insert(ip)
		if err != nil {
			numfail++
			continue
		}
		numinserts += num
	}
	return
}

func ScanIP(cfg *config.Config) (err error) {
	log.Println("Starting set of rdns")
	for {
		var orphans = []IP{}
		cfg.Db.Where("rdns IS NULL").Asc("ip").Find(&orphans)
		if len(orphans) > 0 {
			for _, i := range orphans {
				reverse := i.UpdateRDNS()
				if reverse == "" {
					fmt.Printf("Set \"none\" rdns to IP %s\n", i.IP)
					i.Rdns.String = "none"
				} else {
					fmt.Printf("%s %s\n", i.IP, reverse)
					i.Rdns.String = reverse
				}
				i.Rdns.Valid = true
				_, err = cfg.Db.ID(i.ID).Cols("rdns").Update(&i)
				if err != nil {
					log.Println(err)
				}
			}
		} else {
			break
		}
	}
	log.Println("End set of rdns")
	return
}

// IP describe IP objects
type IP struct {
	ID      int            `xorm:"pk autoincr" json:"-"`
	IP      string         `xorm:"text notnull unique(ipsrc)" json:"ip"`
	Rdns    sql.NullString `xorm:"text default" json:"rdns"`
	Src     string         `xorm:"text notnull unique(ipsrc)" json:"src"`
	Created time.Time      `xorm:"created notnull" json:"-"`
	Updated time.Time      `xorm:"updated notnull" json:"-"`
}
