package ipblws

import (
	"fmt"
	"log"
	"net/http"
	"strconv"

	"git.paulbsd.com/paulbsd/ipbl/src/config"
	"git.paulbsd.com/paulbsd/ipbl/src/ipbl"

	"github.com/labstack/echo/v4"
)

// RunServer runs the main echo HTTP server
func RunServer(cfg *config.Config) (err error) {
	e := echo.New()

	e.HideBanner = cfg.Options.HideBanner

	e.GET("/", func(c echo.Context) error {
		return c.HTML(http.StatusOK, `<html>
		<body style="background-color: black">
		<p style="color:white">Welcome to ipbl software (https://git.paulbsd.com/paulbsd/ipbl)</p>
		</body>
		</html>`)
	})
	e.GET("/ip/:ip", func(c echo.Context) (err error) {
		var ip = ipbl.IP{IP: c.Param("ip")}
		res, _ := cfg.Db.Get(&ip)
		if res {
			return c.JSON(http.StatusOK, ip)
		}
		msg := fmt.Sprintf("IP %s not found", c.Param("ip"))
		log.Println(msg)
		return c.JSON(http.StatusNotFound, msg)
	})
	e.POST("/ip", func(c echo.Context) (err error) {
		var ip = new(ipbl.IP)
		err = c.Bind(ip)
		if err != nil {
			return c.JSON(http.StatusInternalServerError, "Error when parsing body")
		}
		num, err := ip.InsertIP(cfg)
		if err != nil {
			return c.JSON(http.StatusInternalServerError, "Error inserting data")
		}
		msg := fmt.Sprintf("Inserted %d IP", num)
		log.Println(msg)
		return c.JSON(http.StatusOK, msg)
	})
	e.GET("/ips", func(c echo.Context) (err error) {
		var ips = []ipbl.IP{}
		var limit int = 50
		if c.QueryParam("limit") != "" {
			limit, err = strconv.Atoi(c.QueryParam("limit"))
			if err != nil {
				limit = 50
			}
		}
		cfg.Db.Limit(limit).Desc("created").Find(&ips)
		return c.JSON(http.StatusOK, ips)
	})
	e.POST("/ips", func(c echo.Context) (err error) {
		var ips = []ipbl.IP{}
		err = c.Bind(&ips)
		if err != nil {
			return c.JSON(http.StatusInternalServerError, "Error when parsing body")
		}
		numinsert, numfail, _ := ipbl.InsertIPBulk(cfg, &ips)
		msg := fmt.Sprintf("Inserted %d IP, %d errors", numinsert, numfail)
		log.Println(msg)
		return c.JSON(http.StatusOK, msg)
	})

	e.Logger.Fatal(
		e.Start(
			fmt.Sprintf(":%d",
				cfg.Switchs.Port)))

	return
}
